"""Function(s) for getting a format string from a date string."""

import re
from itertools import product
from collections import defaultdict, Counter
from typing import Dict, Iterator, List
from dateutil.parser import parse, ParserError  # type: ignore

FORMAT_DIRECTIVES = [
    '%a', '%A', '%w', '%d', '%b', '%B', '%m', '%y', '%Y',
    '%H', '%I', '%p', '%M', '%S', '%f', '%z', '%Z', '%j',
    '%U', '%W', '%c', '%x', '%X', '%%',
    '%-d', '%-m', '%-H', '%-I', '%-M', '%-S', '%-j']

DATE_TOKEN_REGEX = re.compile(r'([\w]+)|([\W]+)')


# pylint: disable=too-many-locals
def get_all_format_strings(date_string, allow_reuse=False,
                           no_zero=False) -> Iterator[str]:
    """Yield date format strings which match date_string."""
    try:
        parsed_datetime = parse(date_string)
    except ParserError:
        yield ''
    tokens = DATE_TOKEN_REGEX.findall(date_string)
    possible_matches = defaultdict(list)  # type: Dict[str, List[str]]
    for test_format in FORMAT_DIRECTIVES:
        test_string = parsed_datetime.strftime(test_format)
        possible_matches[test_string.lower()].append(test_format)
        possible_parts = []
        for (token, delimiter) in tokens:
            if token:
                if token.lower() in possible_matches:
                    possible_parts.append(
                        possible_matches[token.lower()])
            else:
                possible_parts.append([delimiter])

        possible_combinations = list(product(*possible_parts))
        for parts_sequence in possible_combinations:
            if not no_zero and any(['%-' in x for x in parts_sequence]):
                without_zero = [x.replace('%-', '%') for x in parts_sequence]
                if without_zero in possible_combinations:
                    continue
            if not allow_reuse:
                counts = Counter(parts_sequence)
                max_count = max(
                    counts[diretive] for diretive in FORMAT_DIRECTIVES)
                if max_count > 1:
                    continue
            format_string = "".join(parts_sequence)
            if parsed_datetime.strftime(
                    format_string).lower() == date_string.lower():
                yield format_string


def get_unique_format_strings(date_string, allow_reuse=False) -> List[str]:
    """Return date format strings matching date_string, without duplicates."""
    return list(set(get_all_format_strings(date_string, allow_reuse)))
